import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';

typedef OnWaitingPluggyResponseChangeCallback = void Function(bool isWaiting);

class OauthWebView extends StatefulWidget {
  const OauthWebView(
      {Key? key,
      required this.url,
      required this.isWaitingPluggyResponse,
      required this.onWaitingPluggyResponseChange,
      required this.onOauthCallback})
      : super(key: key);

  final String url;
  final bool isWaitingPluggyResponse;
  final OnWaitingPluggyResponseChangeCallback onWaitingPluggyResponseChange;
  final VoidCallback onOauthCallback;

  @override
  State<StatefulWidget> createState() => OauthWebViewState();
}

class OauthWebViewState extends State<OauthWebView> {
  InAppWebViewController? _inAppWebViewController;

  void _onWebViewCreated(InAppWebViewController controller) {
    setState(() {
      _inAppWebViewController = controller;
    });
    controller.addJavaScriptHandler(
        handlerName: 'pluggyConnectOauthHandler',
        callback: (args) async {
          dynamic dataPayload;

          try {
            dataPayload = jsonDecode(args[0] as String);
          } catch (error) {
            return;
          }

          final String type = dataPayload['type'];
          final dynamic message = dataPayload['message'] ?? '';

          if (type == 'OAUTH_CALLBACK') {
            await _sendMessageToConnect(message);
            widget.onOauthCallback();
            return;
          }
        });
  }

  void _onLoadStop(InAppWebViewController controller, Uri? uri) async {
    await controller.evaluateJavascript(source: '''
      window.opener = {
        postMessage: (message) => {
          window.flutter_inappwebview.callHandler("pluggyConnectOauthHandler", JSON.stringify({type: 'OAUTH_CALLBACK', message}));
        }
      }
      true; // note: this is required, or you'll sometimes get silent failures
    ''');
  }

  void _onUpdateVisitedHistory(InAppWebViewController controller, Uri? uri, _) {
    final bool isPluggyUrl =
        RegExp(r'^\/connectors\/[0-9]+\/oauth\/callback\?$')
            .hasMatch(uri.toString());

    if (isPluggyUrl && !widget.isWaitingPluggyResponse) {
      widget.onWaitingPluggyResponseChange(true);
    }
    if (!isPluggyUrl && widget.isWaitingPluggyResponse) {
      widget.onWaitingPluggyResponseChange(false);
    }
  }

  _sendMessageToConnect(Map<String, dynamic> oauthResult) async {
    final oauthResultMessage = {
      ...oauthResult,
      'source': 'connector-oauth-login'
    };
    await _inAppWebViewController?.evaluateJavascript(source: '''
      (function() {
        document.dispatchEvent(new MessageEvent('message', {
          data: ${jsonEncode(oauthResultMessage)}
        }));
      })();
    ''');
  }

  // We need to override the go back default behaivior
  Future<bool> _backHandlerAndroid() async {
    if (Platform.isIOS) {
      return true;
    }

    await _sendMessageToConnect(
        {'error': 'User navigated back from Oauth WebView'});
    widget.onOauthCallback();
    return false;
  }

  void _backHandlerIOS(DragUpdateDetails details) async {
    if (Platform.isAndroid) {
      return;
    }
    //set the sensitivity for your ios gesture anywhere between 10-50 is good
    const int sensitivity = 25;

    if (details.delta.dx > sensitivity) {
      await _sendMessageToConnect(
          {'error': 'User navigated back from Oauth WebView'});
      //SWIPE FROM RIGHT DETECTION
      widget.onOauthCallback();
    }
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: _backHandlerAndroid,
        child: GestureDetector(
          onHorizontalDragUpdate: _backHandlerIOS,
          child: SafeArea(
              child: InAppWebView(
            initialUrlRequest: URLRequest(url: Uri.parse(widget.url)),
            onWebViewCreated: _onWebViewCreated,
            onLoadStop: _onLoadStop,
            onUpdateVisitedHistory: _onUpdateVisitedHistory,
            initialOptions: InAppWebViewGroupOptions(
                crossPlatform:
                    InAppWebViewOptions(cacheEnabled: false, incognito: true),
                android: AndroidInAppWebViewOptions(
                    thirdPartyCookiesEnabled: false,
                    cacheMode: AndroidCacheMode.LOAD_NO_CACHE)),
          )),
        ));
  }
}
